/******************************************************************************/
/* AEL Sarl : Web hosting, datacenter, gestion et comptabilité déportées      */
/* Copyright (C) 2008 Affaires En Ligne S.A.R.L.                              */
/* Contact : contact@affaires-en-ligne.fr                                     */
/* Web : http://www.spidboutic.fr                                             */
/*                                                                            */
/* This program is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU General Public License                */
/* as published by the Free Software Foundation; either version 2             */
/* of the License, or (at your option) any later version.                     */
/*                                                                            */
/* This program is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the               */
/* GNU General Public License for more details.                               */
/*                                                                            */
/* You should have received a copy of the GNU General Public License          */
/* along with this program; if not, write to the Free Software                */
/* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/******************************************************************************/

import java.io.Console;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Formatter;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;

import org.apache.tools.ant.DefaultLogger;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.ProjectHelper;

public class Installeur {

	private final static double MIN_VERSION = 1.6;
	
	private final String INSTALLEUR_OSAMES_DIR = "OSAMES";
	private final String INSTALLEUR_PROPERTIES_FILE  = "installeur.properties";
	private final String INSTALLEUR_BUILD_FILE  = "installeur.xml";
	
	private Console shell;
	private Properties prop;
	private Map<String,Object>settings;
	
	private String errmsg = "";
	private String ouiAlias;
	private String nonAlias;
	
	private String dbRoot;
	private String dbRootPassword;
	private String osamesDb;
	private String osamesUser;
	private String osamesUserPassword;
	private String webappsDir;
	private String osamesServlet;
	private String servletPath;
	private boolean chargerCommunes;
	private String serveurSmtp;
	private String expediteur;
	
	private String sep;
	private ResourceBundle messages = null;
	
	public static void main(String[] args) {
		
		String[] nums = System.getProperty("java.version").split("\\.");
		if (Double.parseDouble(nums[0]+"."+nums[1]) < MIN_VERSION) {
			System.out.println("Incompatible Java version. You need, at least, version " + MIN_VERSION);
			System.exit(-1); 
		}
		
		Installeur inst = new Installeur();
		try {
			inst.initParams();
			if (!inst.verifierParametres()) {
				System.out.println("\n" + inst.errmsg);
				System.exit(1);
			}
			if (inst.resumer()) {
				inst.installer(inst.creerProjet());
			}
			System.exit(0);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public Installeur() {
		sep = System.getProperty("file.separator");
	}
	
	public Installeur(String adminLogin_, String adminPassword_, String webappsDir_,
			          String osamesServlet_, String osamesDb_, String osamesDbUser_, String osamesDbPassword_,
			          String smtp_, String expediteur_, boolean chargerCommunes_) {
		this();
		dbRoot = adminLogin_;
		dbRootPassword = adminPassword_;
		webappsDir = webappsDir_;
		osamesServlet = osamesServlet_;
		osamesDb = osamesDb_;
		osamesUser = osamesDbUser_;
		osamesUserPassword = osamesDbPassword_;
		chargerCommunes = chargerCommunes_;
		servletPath = webappsDir + sep + osamesServlet;
		serveurSmtp = smtp_;
		expediteur = expediteur_;
	}
	
	public Installeur(Map settings_) {
		this();
		this.prop = (Properties)settings_.get("prop");
		this.messages = (ResourceBundle)settings_.get("messages");
		dbRoot = (String)settings_.get("txtSgbdAdmin");
		dbRootPassword = (String)settings_.get("txtSgbdAdminPassword");
		webappsDir = (String)settings_.get("txtWebapps");
		osamesServlet = (String)settings_.get("txtServlet");
		osamesDb = (String)settings_.get("txtDb");
		osamesUser = (String)settings_.get("txtUser");
		osamesUserPassword = (String)settings_.get("txtPassword");
		serveurSmtp = (String)settings_.get("txtSmtp");
		expediteur = (String)settings_.get("txtExpediteur");
		chargerCommunes = ((Boolean)settings_.get("rbChargerCommunes")).booleanValue();
		servletPath = (String)settings_.get("servletPath");
	}
	
	private void initParams() {
		
		prop = new Properties();
		
		this.shell = System.console();
		
		try {
			prop.load(ClassLoader.getSystemResourceAsStream(INSTALLEUR_PROPERTIES_FILE));
		} catch (Exception e) {
			System.out.println(messages.getString("erreur.proprietes"));
			System.out.println();
			e.printStackTrace();
			System.exit(1);
		}

		String[] langueAbbrev = prop.getProperty("langues.abbreviations").split(" ");
		String[] langueComplet = prop.getProperty("langues.completes").split(" ");

		StringBuilder sb = new StringBuilder("\nHere are the languages supported by this installer:\n");
		Formatter formateur = new Formatter(sb);
		Hashtable<String,Boolean>listeNumLangue = new Hashtable<String,Boolean>();
		int i;
		for (i = 0; i < langueAbbrev.length; i++) {
			listeNumLangue.put(""+(i+1), true);
			formateur.format("%d %-13s", i+1, langueComplet[i]);
			if((i+1) % 3 == 0)
				formateur.format("\n");
		}
		System.out.println(sb.toString());
		int numLangue = Integer.parseInt(promptVariableDansListe("Please select the installation language number in the above list", "1", listeNumLangue));
		String[] langue = langueAbbrev[numLangue - 1].split("_");
		messages = ResourceBundle.getBundle("locale.MessageBundle", new Locale(langue[0], langue[1]), this.getClass().getClassLoader());
		
		ouiAlias = messages.getString("osames.oui.alias").split(" ")[0];
		nonAlias = messages.getString("osames.non.alias").split(" ")[0];
		
		osamesServlet = promptVariable(messages.getString("prompt.servlet.nom"), "OSAMES");
		osamesDb = promptVariable(messages.getString("prompt.base.nom"), "OSAMES");
		osamesUser = promptVariable(messages.getString("prompt.base.user"), "OSAMES");
		osamesUserPassword = promptVariable(messages.getString("prompt.base.password"), "OSAMES");
		webappsDir = promptVariable(messages.getString("prompt.tomcat.base"), "/var/lib/tomcat5.5/webapps");
		dbRoot = promptVariable(messages.getString("prompt.sgbd.admin"), "root");
		dbRootPassword = String.valueOf(shell.readPassword(messages.getString("prompt.sgbd.password")));
		Hashtable<String,Boolean> listeOuiNon = chargerListeOuiNon();
		chargerCommunes = listeOuiNon.get(promptVariableDansListe(messages.getString("prompt.load.communes"), ouiAlias, listeOuiNon));
		
		servletPath = webappsDir + sep + osamesServlet;
	}
	
	public Project creerProjet() throws Exception {
		Project projet = new Project();
		
		// Visualisation des logs
		DefaultLogger logeur = new org.apache.tools.ant.DefaultLogger();
	    logeur.setErrorPrintStream(System.err);
		logeur.setOutputPrintStream(System.out);
		logeur.setMessageOutputLevel(Project.MSG_INFO);
		projet.addBuildListener(logeur);
		
		projet.init();
		
		ProjectHelper helper = ProjectHelper.getProjectHelper();
		helper.parse(projet, new File(ClassLoader.getSystemResource(INSTALLEUR_BUILD_FILE).toURI()));
		
		return projet;
	}
	
	private void installer(Project projet) throws Exception {
		
		// Création du servlet
		creerServlet(projet);
		// Copier le fichier de conf adapté
		creerConf(projet);
		// Création de la base
		creerBase(projet);
		// Chargement des communes
		chargerCommunes(projet);
	}
	
	private boolean resumer() {
		System.out.println(messages.getString("prompt.resumer.servlet") + osamesServlet);
		System.out.println(messages.getString("prompt.resumer.base") + osamesDb);
		System.out.println(messages.getString("prompt.resumer.user") + osamesUser);
		System.out.println(messages.getString("prompt.resumer.user.password") + osamesUserPassword);
		System.out.println(messages.getString("prompt.resumer.admin") + dbRoot);
		System.out.println(messages.getString("prompt.resumer.admin.password") + "**********");
		System.out.println(messages.getString("prompt.resumer.webapps") + webappsDir);
		System.out.println(messages.getString("prompt.resumer.charger.communes") + (chargerCommunes ? messages.getString("osames.oui") : messages.getString("osames.non")));
		System.out.println();
		Hashtable<String,Boolean> listeOuiNon = chargerListeOuiNon();
		return listeOuiNon.get(promptVariableDansListe(messages.getString("prompt.installer"), ouiAlias, listeOuiNon));
	}
	
	public void creerConf(Project projet) {
		// Copier le fichier de conf adapté
		String urlSystemDb = prop.getProperty("jdbc.system.url");
		projet.setProperty("osames.os.configfile", INSTALLEUR_OSAMES_DIR + sep + (System.getProperty("os.name").startsWith("Windows") ? "osames.win.cfg" : "osames.unix.cfg"));
		projet.setProperty("osames.dir.configfile", servletPath + sep + "osames.cfg");
		projet.setProperty("osames.uploadDir", servletPath);
		projet.setProperty("jdbc.osames.url", urlSystemDb.substring(0, urlSystemDb.lastIndexOf('/')+1) + osamesDb);
		projet.setProperty("osames.user", osamesUser);
		projet.setProperty("osames.user.password", osamesUserPassword);
		projet.setProperty("osames.mail.host", serveurSmtp);
		projet.setProperty("osames.mail.from", expediteur);
		projet.executeTarget("creer_conf");
	}
	
	public void creerServlet(Project projet) {
		projet.setProperty("osames.dir", servletPath);
		projet.executeTarget("creer_servlet");
		
		/*File dest = new File(webappsDir, osamesServlet);
		File src = new File(ClassLoader.getSystemResource(INSTALLEUR_OSAMES_DIR).toURI());
		copier(src, dest);*/
	}
	
	public void creerBase(Project projet) {
		// Création de la base
		projet.setProperty("osames.db", osamesDb);
		projet.setProperty("db.root", dbRoot);
		projet.setProperty("db.root.password", dbRootPassword);
		projet.setProperty("osames.sql", INSTALLEUR_OSAMES_DIR + sep + "database" + sep + "mysql" + sep);
		projet.executeTarget("creer_base");
	}
	
	public void chargerCommunes(Project projet) {
		// Chargement des communes
		if (chargerCommunes) {
			projet.executeTarget("charger_communes");
		}
	}
	
	private Hashtable<String,Boolean> chargerListeOuiNon() {
		Hashtable<String,Boolean> liste = new Hashtable<String,Boolean>();
		
		liste.put(messages.getString("osames.oui"), true);
		liste.put(messages.getString("osames.non"), false);
		String[] ouiAliases = messages.getString("osames.oui.alias").split(" ");
		String[] nonAliases = messages.getString("osames.non.alias").split(" ");
		for (int i = 0; i < ouiAliases.length; i++) {
			liste.put(ouiAliases[i], true);
		}
		for (int i = 0; i < nonAliases.length; i++) {
			liste.put(nonAliases[i], false);
		}

		return liste;
	}
	
	private String promptVariable(String prompt, String defaut) {
		
		String valeur = shell.readLine(prompt + " [" + defaut + "] : ");
		valeur = (valeur.equals("") ? defaut : valeur);
		
		return valeur;
	}
	
	private String promptVariableDansListe(String prompt, String defaut, Hashtable<String,Boolean> permises) {
		
		boolean ok = false;
		String valeur = "";
		System.out.println(valeur);
		while (!ok) {
			valeur = shell.readLine(prompt + " [" + defaut + "] : ");
			valeur = (valeur.equals("") ? defaut : valeur);
			if (permises.containsKey(valeur.toLowerCase())) {
				ok = true;
			} else {
				System.out.println(messages == null ? "Wrong value" : messages.getString("erreur.saisie"));
			}
		}
		return valeur;
	}
	
	private boolean verifierParametres() throws ClassNotFoundException {
		return verifierRootPassword() && verifierConnexion() && verifierWebapps();
	}
	
	private boolean verifierRootPassword() {
		if (dbRootPassword.equals("")) {
			errmsg += messages.getString("erreur.password") + "\n";
			return false;
		}
		return true;
	}
	
	private boolean verifierConnexion() throws ClassNotFoundException {
		try {
			Class.forName(prop.getProperty("jdbc.system.driver"));
			Connection con = DriverManager.getConnection(prop.getProperty("jdbc.system.url"), dbRoot, dbRootPassword);
			con.close();
		} catch (SQLException sqle) {
			errmsg += messages.getString("erreur.connexion") + "\n";
			return false;
		}
		return true;
	}
	
	private boolean verifierWebapps() {
		if (!new File(webappsDir).isDirectory()) {
			errmsg += messages.getString("erreur.webapps") + "\n";
			return false;
		}
		return true;
	}
	
	private void copier(File src, File dest) throws IOException {
		
		if (src.isDirectory()) {
			if (!dest.exists()) {
				dest.mkdir();
			}
			
			String fichiers[] = src.list();
			  
		    for(int i = 0; i < fichiers.length; i++) {
		        copier(new File(src, fichiers[i]), new File(dest, fichiers[i]));
		    }
		} else {
	       InputStream in = new FileInputStream(src);
	       OutputStream out = new FileOutputStream(dest); 

	       byte[] buffer = new byte[2048];
           int nb;
	 
           while ((nb = in.read(buffer)) > 0) {
	          out.write(buffer, 0, nb);
           }
	 
	       in.close();
           out.close();

	    }
	}
	
}
